from syntaxtrees.ParseNode import *
from syntaxtrees.AST import *


globalvars = {}       # We will store the calculator's variables here

def lookup(map, name):
    for x,v in map:
        if x==name: return v
    if name not in globalvars.keys(): print 'Undefined:', name
    return globalvars.get(name, 0)


from string import *
import re
from yappsrt import *

class FormulaParserScanner(Scanner):
    patterns = [
        ("'='", re.compile('=')),
        ('[ \r\t\n]+', re.compile('[ \r\t\n]+')),
        ('int', re.compile('[0-9]+')),
        ('real', re.compile('([0-9]+[.][0-9]*)|([0-9]*[.][0-9]+)')),
        ('number', re.compile('([0-9]+)|(([0-9]+[.][0-9]*)|([0-9]*[.][0-9]+))')),
        ('char', re.compile('[a-zA-Z]')),
        ('string', re.compile('[0-9]*[a-zA-Z]+([a-zA-Z]|[0-9])*')),
        ('ref', re.compile('[$]?[A-Z][A-Z]?[A-Z]?[$]?[1-9][0-9]?[0-9]?[0-9]?')),
        ('rangeref', re.compile('[$]?[A-Z][A-Z]?[A-Z]?[$]?[1-9][0-9]?[0-9]?[0-9]?[:][$]?[A-Z][A-Z]?[A-Z]?[$]?[1-9][0-9]?[0-9]?[0-9]?')),
        ('addop', re.compile('[+-]')),
        ('mulop', re.compile('[*/]')),
        ('add', re.compile('\\+')),
        ('sub', re.compile('-')),
        ('mul', re.compile('\\*')),
        ('div', re.compile('/')),
        ('expop', re.compile('\\^')),
        ('funcname', re.compile('[a-zA-Z]+')),
        ('lpar', re.compile('\\(')),
        ('rpar', re.compile('\\)')),
        ('comma', re.compile(',')),
        ('equal', re.compile('=')),
        ('empty', re.compile('')),
        ('end', re.compile(';')),
        ('NUM', re.compile('[0-9]+')),
        ('VAR', re.compile('[a-zA-Z_]+')),
    ]
    def __init__(self, str):
        Scanner.__init__(self,None,['[ \r\t\n]+'],str)

class FormulaParser(Parser):
    def GOAL(self):
        _token_ = self._peek('string', "'='")
        if _token_ == "'='":
            FORMULA = self.FORMULA()
            return FORMULA
        else:# == 'string'
            string = self._scan('string')
        end = self._scan('end')
        return ParseNode("string", string)

    def EXP(self):
        TERM = self.TERM()
        parseNode=ParseNode("exp")
        parseNode.children.append(TERM)
        while self._peek('add', 'sub', 'rpar', 'comma', 'end') in ['add', 'sub']:
            _token_ = self._peek('add', 'sub')
            if _token_ == 'add':
                add = self._scan('add')
                TERM = self.TERM()
                parseNode.children.append(ParseNode("add"))
                parseNode.children.append(TERM)
            else:# == 'sub'
                sub = self._scan('sub')
                TERM = self.TERM()
                parseNode.children.append(ParseNode("sub"))
                parseNode.children.append(TERM)
        return parseNode

    def TERM(self):
        EXPTERM = self.EXPTERM()
        parseNode=ParseNode("term")
        parseNode.children.append(EXPTERM)
        while self._peek('mul', 'div', 'add', 'sub', 'rpar', 'comma', 'end') in ['mul', 'div']:
            _token_ = self._peek('mul', 'div')
            if _token_ == 'mul':
                mul = self._scan('mul')
                EXPTERM = self.EXPTERM()
                parseNode.children.append(ParseNode("mul"))
                parseNode.children.append(EXPTERM)
            else:# == 'div'
                div = self._scan('div')
                EXPTERM = self.EXPTERM()
                parseNode.children.append(ParseNode("div"))
                parseNode.children.append(EXPTERM)
        return parseNode

    def EXPTERM(self):
        SIGNEDFACTOR = self.SIGNEDFACTOR()
        parseNode=ParseNode("expterm")
        parseNode.children.append(SIGNEDFACTOR)
        while self._peek('expop', 'mul', 'div', 'add', 'sub', 'rpar', 'comma', 'end') == 'expop':
            expop = self._scan('expop')
            SIGNEDFACTOR = self.SIGNEDFACTOR()
            parseNode.children.append(ParseNode("exponent"))
            parseNode.children.append(SIGNEDFACTOR)
        return parseNode

    def SIGNEDFACTOR(self):
        _token_ = self._peek('sub', 'lpar', 'int', 'real', 'ref', 'funcname')
        if _token_ != 'sub':
            FACTOR = self.FACTOR()
            return FACTOR
        else:# == 'sub'
            sub = self._scan('sub')
            FACTOR = self.FACTOR()
            parseNode=ParseNode("signedfactor")
            parseNode.children.append(FACTOR)
            return parseNode

    def FACTOR(self):
        _token_ = self._peek('lpar', 'int', 'real', 'ref', 'funcname')
        if _token_ == 'lpar':
            lpar = self._scan('lpar')
            EXP = self.EXP()
            rpar = self._scan('rpar')
            return EXP
        elif _token_ == 'int':
            int = self._scan('int')
            return ParseNode("number",int)
        elif _token_ == 'real':
            real = self._scan('real')
            return ParseNode("number",real)
        elif _token_ == 'ref':
            ref = self._scan('ref')
            return ParseNode("ref", ref)
        else:# == 'funcname'
            FUNCTION = self.FUNCTION()
            return FUNCTION

    def ARG(self):
        parseNode=ParseNode("arg")
        _token_ = self._peek('rangeref', 'sub', 'lpar', 'int', 'real', 'ref', 'funcname')
        if _token_ == 'rangeref':
            rangeref = self._scan('rangeref')
            parseNode.children.append(ParseNode("rangeref", rangeref))
        else:# in ['sub', 'lpar', 'int', 'real', 'ref', 'funcname']
            EXP = self.EXP()
            parseNode.children.append(EXP)
        while self._peek('comma', 'rpar') == 'comma':
            comma = self._scan('comma')
            _token_ = self._peek('rangeref', 'sub', 'lpar', 'int', 'real', 'ref', 'funcname')
            if _token_ == 'rangeref':
                rangeref = self._scan('rangeref')
                parseNode.children.append(ParseNode("rangeref", rangeref))
            else:# in ['sub', 'lpar', 'int', 'real', 'ref', 'funcname']
                EXP = self.EXP()
                parseNode.children.append(EXP)
        return parseNode

    def FUNCTION(self):
        funcname = self._scan('funcname')
        parseNode=ParseNode("function", funcname)
        ARG=None
        lpar = self._scan('lpar')
        if self._peek('rangeref', 'rpar', 'sub', 'lpar', 'int', 'real', 'ref', 'funcname') != 'rpar':
            ARG = self.ARG()
        if ARG!=None:parseNode.children.append(ARG)
        rpar = self._scan('rpar')
        return parseNode

    def FORMULA(self):
        self._scan("'='")
        EXP = self.EXP()
        return EXP


def parse(rule, text):
    P = FormulaParser(FormulaParserScanner(text))
    return wrap_error_reporter(P, rule)



def getCellRef(ref):
    absoluteColumn=0
    absoluteRow=0
    column=""
    row=""
    i=0
    #Checking if the column is absolute
    if ref[i]=='$':
        absoluteColumn=1
        i+=1
    while not ((ref[i]>='0' and ref[i]<='9') or ref[i]=='$'):
        column+=ref[i]
        i+=1

    column=stringToInt(column)

    #Checking if the row is absolute
    if ref[i]=='$':
        absoluteRow=1
        i+=1
    while i<len(ref):
        row+=ref[i]
        i+=1
    row=int(row)

    return CellRef(column, row, absoluteColumn,absoluteRow)

def constructAST(parseTree):
    if parseTree.kind=="number":
        return Number(float(parseTree.strData))
    elif parseTree.kind=="ref":
        return getCellRef(parseTree.strData)
    elif parseTree.kind=="rangeref":
        refs=parseTree.strData.split(":")
        return RangeRef(getCellRef(refs[0]),
                        getCellRef(refs[1]))
    elif parseTree.kind=="function":
        args=[]
        if len(parseTree.children)>0:
            args=constructAST(parseTree.children[0])
        return Function(parseTree.strData, args)
    elif parseTree.kind=="arg":
        args=[]
        for arg in parseTree.children:
            args.append(constructAST(arg))
        return args
    elif parseTree.kind=="exp":
        i=0
        if len(parseTree.children)==1:
            return constructAST(parseTree.children[0])
        else:
            operands=[]
            while i<len(parseTree.children):
                if i==0:
                    operands.append(constructAST(parseTree.children[i]))
                    i+=2
                elif parseTree.children[i-1].kind=="add":
                    operands.append(constructAST(parseTree.children[i]))
                    i+=2
                else:
                    operands.append(Function("invSum", [constructAST(parseTree.children[i])]))
                    i+=2
            return Function("sum", operands)
    elif parseTree.kind=="term":
        i=0
        if len(parseTree.children)==1:
            return constructAST(parseTree.children[0])
        else:
            operands=[]
            while i<len(parseTree.children):
                if i==0:
                    operands.append(constructAST(parseTree.children[i]))
                    i+=2
                elif parseTree.children[i-1].kind=="mul":
                    operands.append(constructAST(parseTree.children[i]))
                    i+=2
                else:
                    operands.append(Function("invMul", [constructAST(parseTree.children[i])]))
                    i+=2
            return Function("mul", operands)
    elif parseTree.kind=="expterm":
        i=0
        if len(parseTree.children)==1:
            return constructAST(parseTree.children[0])
        else:
            operands=[]
            while i<len(parseTree.children):
                operands.append(constructAST(parseTree.children[i]))
                i+=2
            return Function("exp", operands)
    elif parseTree.kind=="signedfactor":
        return Function("invSum", [constructAST(parseTree.children[0])])
    else:
        raise ValueError("Wrong type for simple expression " + parseTree.kind)


def getAST(stringToParse):

    parseTree=parse("GOAL", stringToParse)

    if parseTree[0]:
        return (1,constructAST(parseTree[1]),parseTree[2])
    else:
        return (0,None,parseTree[2])



    #return constructAST(parseTree)

if __name__=='__main__':
    print 'Welcome to the formula parser!'
    # We could have put this loop into the parser, by making the
    # `goal' rule use (expr | set var expr)*, but by putting the
    # loop into Python code, we can make it interactive (i.e., enter
    # one expression, get the result, enter another expression, etc.)
    while 1:
        s = raw_input('>>> ')
        print s
        parse('GOAL', s)