from formulaparser.Parser import *
from formulaparser.syntaxtrees.ASTVisitor import *
from formulaparser.syntaxtrees.AST import *

import unittest
import sys

from math import pow
from math import fabs

class ASTSimpleEvalVisitor(ASTVisitor):
    def __init__(self):
        self.answer=0
        self.stack=[]

    def visitNumber(self, node):
        self.stack.append(node.getValue())

    def visitFunction(self, node):
        if node.getName() =="sum":
            answer=0
            for i in xrange(len(node.getArgs()),0,-1):
                answer+=self.stack[-i]
            self.stack=self.stack[0:-len(node.getArgs())]
            self.stack.append(answer)
        elif node.getName() == "mul":
            answer=1
            for i in xrange(len(node.getArgs()),0,-1):
                answer*=self.stack[-i]
            self.stack=self.stack[0:-len(node.getArgs())]
            self.stack.append(answer)
        elif node.getName() == "exp":
            answer=None
            for i in xrange(len(node.getArgs()),0,-1):
                if answer==None:
                    answer=self.stack[-i]
                else:
                    answer=pow(answer,self.stack[-i])
            self.stack=self.stack[0:-len(node.getArgs())]
            self.stack.append(answer)
        elif node.getName() == "invSum":
            self.stack.append(self.stack.pop()*-1)
        elif node.getName() == "invMul":
            self.stack.append(1/self.stack.pop())
        else:
            raise ValueError, "Unexpected function name: "+node.getName()

    def visitCellRef(self, astNode):
        raise NotImplementedError, "Unexpected AST node, CellRef"

    def visitRangeRef(self, astNode):
        raise NotImplementedError, "Unexpected AST node, RangeRef"

class ParserSuccess(unittest.TestCase):
    """Testing various patterns that should be recognized
    """

    simpleExprSamples = [("=-((4))+3+7+2-1;", 7),
                         ("=-4;", -4.0),
                         ("=4.324;", 4.324),
                         ("=(4);",4),
                         ("=4+3;", 7),
                         ("=4-3;",1),
                         ("=(4-3);",1),
                         ("=4*3;",12),
                         ("=12/3;",4),
                         ("=5^2;",25),
                         ("=4+3-1+2*5;",16),
                         ("=(4+3-1+2)*5;",40),
                         ("=(4+3)*(2+1);", 21),
                         ("=(4-2)*2+(3*2);",10),
                         ("=5+5^2;", 30),
                         ("=(5+5)^2;",100),
                         ("=5^2+3*3;", 34),
                         ("=(2+3)^2;", 25),
                         ("=5^(2+2*2);", 15625),
                         ("=(5-1)^2+4+3+(3+2)^2;", 48),
                         ("=2^(3^2);", 512),
                         ("=2^(3^(2-1));", 8),
                         ("=((((4+3)*2+3+2)+3*3)*2);", 56),
                         ("=((4+(3-2)*5^2));", 29),
                         ("=-2^2^2+3;", 19),
                         ("=1.0/3.0;", 1.0/3.0),
                         ("=2*(4-10)+3^2;", -3),
                         ("=3*4^(2+3)-1^2*3;",3069),
                         ("=2*3*(1-3-(2+3)^2);",-162)
                        ]

    complexExprSamples = [("=MAX(2,3,4,5)/$A$3;",
                           Function("mul",
                             [Function("MAX",
                                [Number(2),
                                 Number(3),
                                 Number(4),
                                 Number(5)
                                ]),
                              Function("invMul",
                                [CellRef(stringToInt("A"),3,1,1)
                                ])
                             ])
                          ),
                          ("=MIN(4+3,2)*2^3;",
                           Function("mul",
                             [Function("MIN",
                               [Function("sum", [Number(4), Number(3)]),
                                Number(2)
                               ]),
                              Function("exp", [Number(2), Number(3)])
                             ])
                          ),
                          ("=A5+4-2*2;",
                            Function("sum",
                              [CellRef(stringToInt("A"),5,0,0),
                               Number(4),
                               Function("invSum",
                                 [Function("mul",
                                    [Number(2), Number(2)])
                                 ])
                             ])
                          ),
                          ("=$A100^2+BB$38*2+MIN(A1:A5);",
                           Function("sum",
                             [Function("exp",
                                [CellRef(stringToInt("A"),100,1,0),
                                 Number(2)
                                ]),
                              Function("mul",
                                [CellRef(stringToInt("BB"),38,0,1),
                                 Number(2)
                                ]),
                              Function("MIN",
                                [RangeRef(CellRef(stringToInt("A"),1,0,0),
                                          CellRef(stringToInt("A"),5,0,0))
                                ])
                             ])
                          )
                         ]

    def equals(self, ast1, ast2):
        if ast1.__class__!=ast2.__class__:
            return 0
        elif isinstance(ast1, Number):
            if ast1.getValue()!=ast2.getValue():
                return 0
        elif isinstance(ast1, Function):
            if ast1.getName()!=ast2.getName():
                return 0
            if len(ast1.getArgs())!=len(ast2.getArgs()):
                return 0
            for i in xrange(len(ast1.getArgs())):
                if not self.equals(ast1.getArgs()[i],
                                   ast2.getArgs()[i]):
                    return 0
        elif isinstance(ast1, CellRef):
            if ast1.getRow()!=ast2.getRow():
                return 0
            if ast1.getColumn()!=ast2.getColumn():
                return 0
            if ast1.isAbsRow()!=ast2.isAbsRow():
                return 0
            if ast1.isAbsColumn()!=ast2.isAbsColumn():
                return 0
        elif isinstance(ast1, RangeRef):
            if not self.equals(ast1.getFirstCorner(),
                               ast2.getFirstCorner()):
                return 0
            if not self.equals(ast1.getSecondCorner(),
                               ast2.getSecondCorner()):
                return 0
        else:
            raise TypeError, "Invalid type for arguments:"+str(ast1.__class__)
        return 1


    def evalSimpleExpressions(self,tree):
        if tree.kind=="number":
            return float(tree.strData)
        elif tree.kind=="exp":
            i=0
            answer=0
            while 1:
                if i==0:
                    answer=self.evalSimpleExpressions(tree.children[i])
                if i+1==len(tree.children):
                    break
                elif tree.children[i+1].kind=="add":
                    answer+=self.evalSimpleExpressions(tree.children[i+2])
                    i+=2
                else:
                    answer-=self.evalSimpleExpressions(tree.children[i+2])
                    i+=2
            return answer
        elif tree.kind=="term":
            i=0
            answer=0
            while 1:
                if i==0:
                    answer=self.evalSimpleExpressions(tree.children[i])
                if i+1==len(tree.children):
                    break
                elif tree.children[i+1].kind=="mul":
                    answer*=self.evalSimpleExpressions(tree.children[i+2])
                    i+=2
                else:
                    answer/=self.evalSimpleExpressions(tree.children[i+2])
                    i+=2
            return answer
        elif tree.kind=="expterm":
            i=0
            answer=0
            while 1:
                if i==0:
                    answer=self.evalSimpleExpressions(tree.children[i])
                if i+1==len(tree.children):
                    break
                else:
                    answer=pow(answer,
                               self.evalSimpleExpressions(tree.children[i+2]))
                    i+=2
            return answer
        elif tree.kind=="signedfactor":
            return -1*self.evalSimpleExpressions(tree.children[0])
        else:
            raise Exception("Wrong type for simple expression" + tree.kind)


    def testParseTreeSimpleExpressionSuccess(self):
        """Testing the parse tree by evaluating simple arithmetic expressions
        """
        for sample in self.simpleExprSamples:
            t=parse("GOAL", sample[0])[1]
            ans=self.evalSimpleExpressions(t)
            self.failIf(fabs(ans-sample[1])>0.00001,
                        str(sample[1]) + " should equal " + str(ans))

    def testASTSimpleExpressionSuccess(self):
        """Testing the AST by evaluating simple arithmetic expressions
        """
        for sample in self.simpleExprSamples:
            t=getAST(sample[0])[1]
            v=ASTSimpleEvalVisitor()
            t.accept(v)
            ans=v.stack[0]
            self.failIf(fabs(ans-sample[1])>0.00001,
                             str(sample[1]) + " should equal " + str(ans))

    def testASTComplexExpressionSuccess(self):
        """Testing that a formula returns the expected AST
        """
        for sample in self.complexExprSamples:
            s1=getAST(sample[0])[1]
            s2=sample[1]
            self.failIf(not self.equals(s1,s2),
                        "A sample did not return the expected AST")

class ParserFailure(unittest.TestCase):
    """Testing various invalid inputs
    """

    def testParserFailure(self):
        """Testing the parser for failure
        """
        pass


if __name__ == "__main__":
    unittest.main()







